package com.atguigu.ssyx.client.product;

import com.atguigu.ssyx.common.result.Result;
import com.atguigu.ssyx.model.product.Category;
import com.atguigu.ssyx.model.product.SkuInfo;
import com.atguigu.ssyx.vo.product.SkuInfoVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

@FeignClient(value = "service-product", path = "/api/product")
public interface ProductFeignClient {
    // 根据分类id获取分类信息
    @GetMapping("inner/getCategory/{categoryId}")
    Result<Category> getCategory(@PathVariable Long categoryId);

    // 根据skuId获取sku信息
    @GetMapping("inner/getSkuInfo/{skuId}")
    Result<SkuInfo> getSkuInfo(@PathVariable Long skuId);

    // 根据skuId得到sku信息列表
    @PostMapping("inner/findSkuInfoList")
    Result<List<SkuInfo>> findSkuInfoList(@RequestBody List<Long> skuIdList);

    // 根据关键字获取sku列表
    @GetMapping("inner/findSkuInfoByKeyword/{keyword}")
    Result<List<SkuInfo>> findSkuInfoByKeyword(@PathVariable("keyword") String keyword);

    // 批量获取分类信息
    @PostMapping("inner/findCategoryList")
    List<Category> findCategoryList(@RequestBody List<Long> rangeIdList);

    // 获取全部商品分类
    @GetMapping("findAllList")
    Result<List<Category>> findAllCategoryList();

    // 获取新人专享
    @GetMapping("inner/findNewPersonSkuInfoList")
    Result<List<SkuInfo>> findNewPersonSkuInfoList();

    // 根据skuId获取sku信息
    @GetMapping("inner/getSkuInfoVo/{skuId}")
    SkuInfoVo getSkuInfoVo(@PathVariable Long skuId);
}
