package com.atguigu.ssyx.acl.service.impl;

import com.atguigu.ssyx.acl.mapper.AdminMapper;
import com.atguigu.ssyx.acl.service.AdminService;
import com.atguigu.ssyx.model.acl.Admin;
import com.atguigu.ssyx.vo.acl.AdminQueryVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import org.springframework.util.StringUtils;

@Service
@Slf4j
public class AdminServiceImpl extends ServiceImpl<AdminMapper, Admin> implements AdminService {
    /**
     * * 条件分页查询
     *
     * @param page  当前页码
     * @param limit 每页记录数
     * @param vo    查询对象
     * @return 分页结果
     */
    @Override
    public IPage<Admin> selectPage(Long page, Long limit, AdminQueryVo vo) {
        Page<Admin> pageParam = new Page<Admin>(page, limit);
        String name = vo.getName();
        String username = vo.getUsername();

        // 封装条件进行返回
        LambdaQueryWrapper<Admin> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(!StringUtils.isEmpty(username), Admin::getUsername, username);
        wrapper.like(!StringUtils.isEmpty(name), Admin::getName, name);

        return baseMapper.selectPage(pageParam, wrapper);
    }

    /**
     * 新增管理用户
     *
     * @param admin 用户管理员
     */
    @Override
    public Boolean saveByAdmin(Admin admin) {
        // 获取密码并加密
        String password = admin.getPassword();
        String encryptPassword = DigestUtils.md5DigestAsHex(password.getBytes());
        // 设置密码
        admin.setPassword(encryptPassword);
        return save(admin);
    }
}