package com.atguigu.ssyx.activity.service.impl;

import com.atguigu.ssyx.activity.mapper.CouponInfoMapper;
import com.atguigu.ssyx.activity.mapper.CouponRangeMapper;
import com.atguigu.ssyx.activity.service.CouponInfoService;
import com.atguigu.ssyx.activity.service.module.CouponInfoServiceModule;
import com.atguigu.ssyx.client.product.ProductFeignClient;
import com.atguigu.ssyx.enums.CouponRangeType;
import com.atguigu.ssyx.model.activity.CouponInfo;
import com.atguigu.ssyx.model.activity.CouponRange;
import com.atguigu.ssyx.model.base.BaseEntity;
import com.atguigu.ssyx.model.order.CartInfo;
import com.atguigu.ssyx.model.product.Category;
import com.atguigu.ssyx.model.product.SkuInfo;
import com.atguigu.ssyx.vo.activity.CouponRuleVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class CouponInfoServiceImpl extends ServiceImpl<CouponInfoMapper, CouponInfo> implements CouponInfoService {
    @Autowired
    private CouponRangeMapper couponRangeMapper;
    @Autowired
    private CouponInfoMapper couponInfoMapper;
    @Autowired
    private ProductFeignClient productFeignClient;
    @Autowired
    private CouponInfoServiceModule module;

    /**
     * * 获取分页列表
     *
     * @param pageParam 查询分页条件
     * @return 分页结果
     */
    @Override
    public IPage<CouponInfo> selectPage(Page<CouponInfo> pageParam) {
        Page<CouponInfo> page = page(pageParam);
        List<CouponInfo> pageRecords = page.getRecords();

        pageRecords.forEach(record -> {
            record.setCouponTypeString(record.getCouponType().getComment());
            CouponRangeType rangeType = record.getRangeType();
            if (rangeType != null) {
                record.setRangeTypeString(rangeType.getComment());
            }
        });
        return page;
    }

    /**
     * 获取优惠券
     *
     * @param id 优惠卷Id
     * @return 优惠卷
     */
    @Override
    public CouponInfo getCouponInfo(Long id) {
        CouponInfo couponInfo = getById(id);
        couponInfo.setCouponTypeString(couponInfo.getCouponType().getComment());

        // 使用范围
        if (couponInfo.getRangeType() != null) {
            couponInfo.setRangeTypeString(couponInfo.getRangeType().getComment());
        }
        return couponInfo;
    }

    /**
     * 获取优惠券信息
     *
     * @param id 优惠卷Id
     * @return Map<String, Object>
     */
    @Override
    public Map<String, Object> findCouponRuleList(Long id) {
        Map<String, Object> result = new HashMap<>();
        CouponInfo couponInfo = getById(id);
        // 根据Id查询 couponRange 列表
        List<CouponRange> couponRangeList = couponRangeMapper.selectList(Wrappers.<CouponRange>lambdaQuery().eq(CouponRange::getCouponId, id));
        List<Long> rangeIdList = couponRangeList.stream().map(CouponRange::getRangeId).collect(Collectors.toList());

        // 分别判断封装不同数据
        if (!CollectionUtils.isEmpty(rangeIdList)) {
            if (couponInfo.getRangeType() == CouponRangeType.SKU) {
                List<SkuInfo> skuInfoList = productFeignClient.findSkuInfoList(rangeIdList).getData();
                result.put("skuInfoList", skuInfoList);
            }
            if (couponInfo.getRangeType() == CouponRangeType.CATEGORY) {
                List<Category> categoryList = productFeignClient.findCategoryList(rangeIdList);
                result.put("categoryList", categoryList);
            }
        }

        return result;
    }

    /**
     * * 新增活动
     *
     * @param vo 新的活动
     */
    @Override
    public void saveCouponRule(CouponRuleVo vo) {
        // 删除之前的内容
        couponRangeMapper.delete(Wrappers.<CouponRange>lambdaQuery().eq(CouponRange::getCouponId, vo.getCouponId()));
        // 查询优惠卷基本信息
        CouponInfo couponInfo = getById(vo.getCouponId());
        couponInfo.setRangeType(vo.getRangeType());
        couponInfo.setConditionAmount(vo.getConditionAmount());
        couponInfo.setAmount(vo.getAmount());
        couponInfo.setConditionAmount(vo.getConditionAmount());
        couponInfo.setRangeDesc(vo.getRangeDesc());
        // 添加优惠卷信息
        updateById(couponInfo);

        // 添加优惠卷新的规则数据
        List<CouponRange> couponRangeList = vo.getCouponRangeList();
        couponRangeList.forEach(couponRange -> {
            couponRange.setCouponId(vo.getCouponId());
            couponRangeMapper.insert(couponRange);
        });
    }

    @Override
    public List<CouponInfo> findCouponByKeyword(String keyword) {
        //  模糊查询
        QueryWrapper<CouponInfo> couponInfoQueryWrapper = new QueryWrapper<>();
        couponInfoQueryWrapper.like("coupon_name", keyword);
        return list(couponInfoQueryWrapper);
    }

    /**
     * * 获取优惠券信息
     *
     * @param skuId  商品id
     * @param userId 用户id
     * @return 优惠卷列表
     */
    @Override
    public List<CouponInfo> findCouponInfo(Long skuId, Long userId) {
        SkuInfo skuInfo = productFeignClient.getSkuInfo(skuId).getData();
        if (null == skuInfo) return new ArrayList<>();
        // 根据skuId获取活动规则数据
        return couponInfoMapper.selectCouponInfoList(skuInfo.getId(), skuInfo.getCategoryId(), userId);
    }

    /**
     * * 购物车可使用的优惠券列表
     *
     * @param cartInfoList 购物车列表信息
     * @param userId       用户ID
     * @return List<CouponInfo>
     */
    @Override
    public List<CouponInfo> findCartCouponInfo(List<CartInfo> cartInfoList, Long userId) {
        // 获取全部用户优惠券
        List<CouponInfo> userAllCouponInfoList = baseMapper.selectCartCouponInfoList(userId);
        if (CollectionUtils.isEmpty(userAllCouponInfoList)) return null;

        // 获取优惠券id列表
        List<Long> couponIdList = userAllCouponInfoList.stream().map(BaseEntity::getId).collect(Collectors.toList());
        // 查询优惠券对应的范围
        List<CouponRange> couponRangesList = couponRangeMapper.selectList(new LambdaQueryWrapper<CouponRange>().in(CouponRange::getCouponId, couponIdList));
        // 获取优惠券id对应的满足使用范围的购物项skuId列表
        Map<Long, List<Long>> couponIdToSkuIdMap = module.findCouponIdToSkuIdMap(cartInfoList, couponRangesList);
        // 优惠后减少金额
        BigDecimal reduceAmount = new BigDecimal("0");
        // 记录最优优惠券
        CouponInfo optimalCouponInfo = null;
        for (CouponInfo couponInfo : userAllCouponInfoList) {
            if (CouponRangeType.ALL == couponInfo.getRangeType()) {
                // 全场通用
                // 判断是否满足优惠使用门槛
                // 计算购物车商品的总价
                BigDecimal totalAmount = computeTotalAmount(cartInfoList);
                if (totalAmount.subtract(couponInfo.getConditionAmount()).doubleValue() >= 0) {
                    couponInfo.setIsSelect(1);
                }
            } else {
                // 优惠券id对应的满足使用范围的购物项skuId列表
                List<Long> skuIdList = couponIdToSkuIdMap.get(couponInfo.getId());
                // 当前满足使用范围的购物项
                List<CartInfo> currentCartInfoList = cartInfoList.stream().filter(cartInfo -> skuIdList.contains(cartInfo.getSkuId())).collect(Collectors.toList());
                BigDecimal totalAmount = computeTotalAmount(currentCartInfoList);
                if (totalAmount.subtract(couponInfo.getConditionAmount()).doubleValue() >= 0) {
                    couponInfo.setIsSelect(1);
                }
            }
            if (couponInfo.getIsSelect() == 1 && couponInfo.getAmount().subtract(reduceAmount).doubleValue() > 0) {
                reduceAmount = couponInfo.getAmount();
                optimalCouponInfo = couponInfo;
            }
        }
        if (null != optimalCouponInfo) {
            optimalCouponInfo.setIsOptimal(1);
        }
        return userAllCouponInfoList;
    }

    private BigDecimal computeTotalAmount(List<CartInfo> cartInfoList) {
        BigDecimal total = new BigDecimal("0");
        for (CartInfo cartInfo : cartInfoList) {
            // 是否选中
            if (cartInfo.getIsChecked() == 1) {
                BigDecimal itemTotal = cartInfo.getCartPrice().multiply(new BigDecimal(cartInfo.getSkuNum()));
                total = total.add(itemTotal);
            }
        }
        return total;
    }
}
