package com.atguigu.ssyx.cart.controller;


import com.atguigu.ssyx.cart.service.CartInfoService;
import com.atguigu.ssyx.client.ActivityFeignClient;
import com.atguigu.ssyx.common.context.BaseContext;
import com.atguigu.ssyx.common.result.Result;
import com.atguigu.ssyx.model.order.CartInfo;
import com.atguigu.ssyx.vo.order.OrderConfirmVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@Api(tags = "购物车相关接口")
@RestController
@RequestMapping("/api/cart")
public class CartInfoController {
    @Autowired
    private CartInfoService cartInfoService;
    @Autowired
    private ActivityFeignClient activityFeignClient;

    @ApiOperation(value = "添加购物车")
    @GetMapping("addToCart/{skuId}/{skuNum}")
    public Result<CartInfo> addToCart(@PathVariable Long skuId, @PathVariable Integer skuNum) {
        Long userId = BaseContext.getUserId();
        cartInfoService.addToCart(skuId, userId, skuNum);
        return Result.success();
    }

    @ApiOperation(value = "删除购物车内容")
    @DeleteMapping("deleteCart/{skuId}")
    public Result<CartInfo> deleteCart(@PathVariable Long skuId) {
        Long userId = BaseContext.getUserId();
        cartInfoService.deleteCart(skuId, userId);
        return Result.success();
    }

    @ApiOperation(value = "清空购物车")
    @DeleteMapping("deleteAllCart")
    public Result<CartInfo> deleteAllCart() {
        Long userId = BaseContext.getUserId();
        cartInfoService.deleteAllCart(userId);
        return Result.success();
    }

    @ApiOperation(value = "批量删除购物车")
    @PostMapping("batchDeleteCart")
    public Result<CartInfo> batchDeleteCart(@RequestBody List<Long> skuIdList) {
        Long userId = BaseContext.getUserId();
        cartInfoService.batchDeleteCart(userId, skuIdList);
        return Result.success();
    }

    @ApiOperation(value = "查询购物车列表")
    @GetMapping("cartList")
    public Result<List<CartInfo>> cartList() {
        Long userId = BaseContext.getUserId();
        List<CartInfo> cartInfoList = cartInfoService.getCartList(userId);
        return Result.success(cartInfoList);
    }

    @ApiOperation(value = "查询带优惠卷的购物车")
    @GetMapping("activityCartList")
    public Result<OrderConfirmVo> activityCartList() {
        Long userId = BaseContext.getUserId();
        List<CartInfo> cartInfoList = cartInfoService.getCartList(userId);
        OrderConfirmVo orderTradeVo = activityFeignClient.findCartActivityAndCoupon(cartInfoList, userId);
        return Result.success(orderTradeVo);
    }

    @ApiOperation(value = "更新选中状态")
    @GetMapping("checkCart/{skuId}/{isChecked}")
    public Result<CartInfo> checkCart(@PathVariable Long skuId, @PathVariable Integer isChecked) {
        Long userId = BaseContext.getUserId();
        cartInfoService.checkCart(userId, isChecked, skuId);
        return Result.success();
    }

    @ApiOperation(value = "更新全部选中")
    @GetMapping("checkAllCart/{isChecked}")
    public Result<CartInfo> checkAllCart(@PathVariable Integer isChecked) {
        Long userId = BaseContext.getUserId();
        cartInfoService.checkAllCart(userId, isChecked);
        return Result.success();
    }

    @ApiOperation(value = "批量选择购物车")
    @PostMapping("batchCheckCart/{isChecked}")
    public Result<CartInfo> batchCheckCart(@RequestBody List<Long> skuIdList, @PathVariable Integer isChecked) {
        Long userId = BaseContext.getUserId();
        cartInfoService.batchCheckCart(skuIdList, userId, isChecked);
        return Result.success();
    }
}

