package com.atguigu.ssyx.product.controller;


import com.atguigu.ssyx.common.result.Result;
import com.atguigu.ssyx.model.product.Attr;
import com.atguigu.ssyx.product.service.AttrService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 商品属性 前端控制器
 * </p>
 *
 * @author bunny
 * @since 2024-04-03
 */
@Api(value = "Attr管理", tags = "平台属性管理")
@RestController
@RequestMapping(value = "/admin/product/attr")
public class AttrController {
    @Autowired
    private AttrService attrService;

    @ApiOperation(value = "获取列表")
    @GetMapping("{attrGroupId}")
    public Result<List<Attr>> index(
            @ApiParam(name = "attrGroupId", value = "分组id", required = true)
            @PathVariable Long attrGroupId) {
        List<Attr> attrList = attrService.findByAttrGroupId(attrGroupId);
        return Result.success(attrList);
    }

    @ApiOperation(value = "获取")
    @GetMapping("get/{id}")
    public Result<Attr> get(@PathVariable Long id) {
        Attr attr = attrService.getById(id);
        return Result.success(attr);
    }

    @ApiOperation(value = "新增")
    @PostMapping("save")
    public Result<Attr> save(@RequestBody Attr attr) {
        attrService.save(attr);
        return Result.success();
    }

    @ApiOperation(value = "修改")
    @PutMapping("update")
    public Result<Attr> updateById(@RequestBody Attr attr) {
        attrService.updateById(attr);
        return Result.success();
    }

    @ApiOperation(value = "删除")
    @DeleteMapping("remove/{id}")
    public Result<Attr> remove(@PathVariable Long id) {
        attrService.removeById(id);
        return Result.success();
    }

    @ApiOperation(value = "根据id列表删除")
    @DeleteMapping("batchRemove")
    public Result<Attr> batchRemove(@RequestBody List<Long> ids) {
        attrService.removeByIds(ids);
        return Result.success();
    }
}

