package com.atguigu.ssyx.user.service.impl;

import com.atguigu.ssyx.client.region.RegionFeignClient;
import com.atguigu.ssyx.enums.UserType;
import com.atguigu.ssyx.model.user.Leader;
import com.atguigu.ssyx.model.user.User;
import com.atguigu.ssyx.model.user.UserDelivery;
import com.atguigu.ssyx.user.mapper.UserMapper;
import com.atguigu.ssyx.user.service.LeaderService;
import com.atguigu.ssyx.user.service.UserDeliveryService;
import com.atguigu.ssyx.user.service.UserService;
import com.atguigu.ssyx.vo.user.LeaderAddressVo;
import com.atguigu.ssyx.vo.user.UserLoginVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {
    @Autowired
    private UserDeliveryService userDeliveryService;
    @Autowired
    private LeaderService leaderService;
    @Autowired
    private RegionFeignClient regionFeignClient;

    /**
     * 先根据openid进行数据库查询
     *
     * @param openId openId
     * @return User
     */
    @Override
    public User getUserByOpenId(String openId) {
        return getOne(Wrappers.<User>lambdaQuery().eq(User::getOpenId, openId));
    }

    /**
     * * 根据用户Id查询默认团长Id
     *
     * @param userId 用户Id
     * @return 团藏地址信息
     */
    @Override
    public LeaderAddressVo getLeaderAddressVoByUserId(Long userId) {
        // 查询团长信息
        LambdaQueryWrapper<UserDelivery> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(userId != null, UserDelivery::getUserId, userId);
        wrapper.eq(UserDelivery::getIsDefault, 1);
        UserDelivery userDelivery = userDeliveryService.getOne(wrapper);
        if (userDelivery == null) return null;
        // 查询团长Id及其其它信息
        Leader leader = leaderService.getById(userDelivery.getLeaderId());

        LeaderAddressVo leaderAddressVo = new LeaderAddressVo();
        BeanUtils.copyProperties(leader, leaderAddressVo);
        leaderAddressVo.setUserId(userId);
        leaderAddressVo.setLeaderId(leader.getId());
        leaderAddressVo.setLeaderName(leader.getName());
        leaderAddressVo.setLeaderPhone(leader.getPhone());
        leaderAddressVo.setWareId(userDelivery.getWareId());
        leaderAddressVo.setStorePath(leader.getStorePath());
        return leaderAddressVo;
    }

    /**
     * * 获取当前用户登录信息
     *
     * @param userId 用户Id
     * @return UserLoginVo
     */
    @Override
    public UserLoginVo getUserLoginVo(Long userId) {
        UserLoginVo userLoginVo = new UserLoginVo();
        User user = this.getById(userId);
        userLoginVo.setNickName(user.getNickName());
        userLoginVo.setUserId(userId);
        userLoginVo.setPhotoUrl(user.getPhotoUrl());
        userLoginVo.setOpenId(user.getOpenId());
        userLoginVo.setIsNew(user.getIsNew());

        // 如果是团长获取当前前团长id与对应的仓库id
        if (user.getUserType() == UserType.LEADER) {
            LambdaQueryWrapper<Leader> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(Leader::getUserId, userId);
            queryWrapper.eq(Leader::getCheckStatus, 1);
            Leader leader = leaderService.getOne(queryWrapper);
            if (null != leader) {
                userLoginVo.setLeaderId(leader.getId());
                Long wareId = regionFeignClient.getWareId(leader.getRegionId());
                userLoginVo.setWareId(wareId);
            }
        } else {
            // 如果是会员获取当前会员对应的仓库id
            LambdaQueryWrapper<UserDelivery> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(UserDelivery::getUserId, userId);
            queryWrapper.eq(UserDelivery::getIsDefault, 1);
            UserDelivery userDelivery = userDeliveryService.getOne(queryWrapper);
            if (null != userDelivery) {
                userLoginVo.setLeaderId(userDelivery.getLeaderId());
                userLoginVo.setWareId(userDelivery.getWareId());
            } else {
                userLoginVo.setLeaderId(1L);
                userLoginVo.setWareId(1L);
            }
        }
        return userLoginVo;
    }
}
